#!/usr/bin/env /opt/opensvc/bin/python
""" 
module uses OSVC_COMP_REMOVE_FILES_... vars

example: 
[
 "/foo/to/delete",
 "/bar/to/delete"
]

"""

import os
import sys
import re
import json
from glob import glob
from subprocess import Popen 
import shutil

sys.path.append(os.path.dirname(__file__))

from comp import *

blacklist = [
 "/root"
]

class CompRemoveFiles(object):
    def __init__(self, prefix='OSVC_COMP_REMOVE_FILES_'):
        self.prefix = prefix.upper()
        patterns = []

        for k in [key for key in os.environ if key.startswith(self.prefix)]:
            try:
                patterns += json.loads(os.environ[k])
            except ValueError:
                print >>sys.stderr, 'list syntax error on var[', k, '] = ', os.environ[k]

        if len(patterns) == 0:
            raise NotApplicable

        patterns = sorted(list(set(patterns)))
        patterns = self.subst_envvars(patterns)
        self.files = self.expand_patterns(patterns)

        if len(self.files) == 0:
            print "no files matching patterns"
            raise NotApplicable

    def expand_patterns(self, patterns):
        l = []
        for pattern in patterns:
            l += glob(pattern)
        return l

    def subst_envvars(self, patterns):
        l = []
        p = re.compile('%%ENV:\w+%%')
        for pattern in patterns:
            if type(pattern) not in [str, unicode]:
                continue
            for m in p.findall(pattern):
                s = m.strip("%").replace('ENV:', '')
                if s in os.environ:
                    v = os.environ[s]
                elif 'OSVC_COMP_'+s in os.environ:
                    v = os.environ['OSVC_COMP_'+s]
                else:
                    print >>sys.stderr, s, 'is not an env variable'
                    raise NotApplicable()
                pattern = pattern.replace(m, v)
            l.append(pattern)
        return l

    def fixable(self):
        return RET_NA

    def check_file(self, _file):
        if not os.path.exists(_file):
            print _file, "does not exist. on target."
            return RET_OK
        print >>sys.stderr, _file, "exists. shouldn't"
        return RET_ERR

    def fix_file(self, _file):
        if not os.path.exists(_file):
            return RET_OK
        try:
            if os.path.isdir(_file) and not os.path.islink(_file):
                shutil.rmtree(_file)
            else:
                os.unlink(_file)
            print _file, "deleted"
        except Exception as e:
            print >>sys.stderr, "failed to delete", _file, "(%s)"%str(e)
            return RET_ERR
        return RET_OK

    def check(self):
        r = 0
        for _file in self.files:
            r |= self.check_file(_file)
        return r

    def fix(self):
        r = 0
        for _file in self.files:
            r |= self.fix_file(_file)
        return r

if __name__ == "__main__":
    syntax = """syntax:
      %s PREFIX check|fixable|fix"""%sys.argv[0]
    if len(sys.argv) != 3:
        print >>sys.stderr, "wrong number of arguments"
        print >>sys.stderr, syntax
        sys.exit(RET_ERR)
    try:
        o = CompRemoveFiles(sys.argv[1])
        if sys.argv[2] == 'check':
            RET = o.check()
        elif sys.argv[2] == 'fix':
            RET = o.fix()
        elif sys.argv[2] == 'fixable':
            RET = o.fixable()
        else:
            print >>sys.stderr, "unsupported argument '%s'"%sys.argv[2]
            print >>sys.stderr, syntax
            RET = RET_ERR
    except NotApplicable:
        sys.exit(RET_NA)
    except:
        import traceback
        traceback.print_exc()
        sys.exit(RET_ERR)

    sys.exit(RET)

